# Exports from nmw_0.1.6 (estimation methods)
export(AddCox)
export(CalcSmat)
export(CombDmExPc)
export(CovStep)
export(DECN)
export(DesclVar)
export(EstStep)
export(InitStep)
export(Mx)
export(Obj)
export(ObjEta)
export(Oi)
export(Oi0)
export(Oi1)
export(OiS0)
export(OiS1)
export(PostHocEta)
export(ScaleVar)
export(SqrtInvCov)
export(TabStep)
export(TrimOut)
export(ltv2mat)
export(mat2ltv)
export(mat2utv)
export(utv2mat)

# Exports from nmw_0.3.0 (NM dataset construction helpers)
export(add_crcl_cg)
export(build_dose_records)
export(build_nm_dataset)
export(build_obs_records)
export(code_race)
export(code_sex)
export(crcl_cg)
export(dat2_time_to_posix)
export(locf_value)
export(merge_cov_locf)
export(nm_to_molar)
export(parse_dtc)

# Exports from nma_0.1.0 (post-run processing)
export(AddDoNoTaLD)
export(AddPage)
export(BtwTagMat)
export(BtwTagVals)
export(ClosePDF)
export(DxPlotPost)
export(EBEpair)
export(ExpandDoseHist)
export(GetAICc)
export(GetCountEps)
export(GetCountEta)
export(GetCountObs)
export(GetCountPara)
export(GetEstMethod)
export(GetOFV)
export(LogDV)
export(LogDV2)
export(MinSuccess)
export(NMIDStat)
export(NMVarStat)
export(Outline)
export(ParseOut)
export(PrepPDF)
export(PrinMTxt)
export(PrinTxt)
export(RemoveNA)
export(ResTest)
export(SESuccess)
export(SumOut)
export(mlr2)
export(nmw_report_ebe)
export(nmw_report_indipk)
export(nmw_report_input)
export(nmw_report_ofv)
export(nmw_report_output)
export(nmw_report_param)
export(nmw_report_pred)
export(nmw_report_resid)
export(nmw_run)
export(run.test.nm)

# Imports
importFrom(numDeriv, grad, hessian)
importFrom(MASS, ginv)
importFrom(grDevices, pdf, dev.off)
importFrom(graphics, plot, lines, text, par, hist, boxplot, pairs, mtext,
           abline, points, rect, screen, split.screen, close.screen, layout,
           strwidth)
importFrom(stats, density, shapiro.test, t.test, anova, lm, formula, cor,
           cov, cov2cor, lowess, qnorm, pnorm, qbinom, pbinom, sd, var,
           pt, qqnorm, setNames, optim, median)
importFrom(utils, read.table, read.csv, read.fwf, write.csv, capture.output,
           head, tail)
