% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetOffDiagOmega}
\alias{GetOffDiagOmega}
\title{Get Off-Diagonal Omega Count}
\usage{
GetOffDiagOmega()
}
\value{
integer
}
\description{
Get Off-Diagonal Omega Count
}
\keyword{internal}
