% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCountFixedTheta}
\alias{GetCountFixedTheta}
\title{Get Count of Fixed Thetas}
\usage{
GetCountFixedTheta()
}
\value{
integer
}
\description{
Get Count of Fixed Thetas
}
\keyword{internal}
