\name{Objs}
\alias{ObjEta}
\alias{Oi}
\alias{Oi0}
\alias{Oi1}
\alias{Obj}
\alias{OiS0}
\alias{OiS1}
\alias{CalcSmat}
\alias{PostHocEta}
\title{Internal Obj Functions}
\description{Internal Obj functions}
\details{These are not to be called by the user.}
\keyword{internal}
