% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_data.R
\name{RmvFixed}
\alias{RmvFixed}
\title{Remove Fixed (Non-Varying) Columns}
\usage{
RmvFixed(Table)
}
\arguments{
\item{Table}{data.frame or matrix}
}
\value{
data.frame with fixed columns removed
}
\description{
Removes columns where all values are the same across rows.
}
\keyword{internal}
