% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdf_report.R
\name{DxPlotPost}
\alias{DxPlotPost}
\title{Diagnostic Plot for Post-Processing}
\usage{
DxPlotPost(x, y, mat, xlbl, ylbl, smooth, xlm = "", ylm = "", Log = "")
}
\arguments{
\item{x}{numeric vector, x-axis values}

\item{y}{numeric vector, y-axis values}

\item{mat}{data.frame with an ID column}

\item{xlbl}{character, x-axis label}

\item{ylbl}{character, y-axis label}

\item{smooth}{character, "T" for lowess smoothing, "F" for identity line}

\item{xlm}{numeric vector of length 2, x-axis limits}

\item{ylm}{numeric vector of length 2, y-axis limits}

\item{Log}{character, log transformation for axes (e.g., "y")}
}
\description{
Creates spaghetti-style diagnostic plots with individual ID labels.
}
