% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_stats.R
\name{AllNA}
\alias{AllNA}
\title{Check if All Values are NA}
\usage{
AllNA(Column)
}
\arguments{
\item{Column}{vector}
}
\value{
logical
}
\description{
Check if All Values are NA
}
\keyword{internal}
