% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCountUnfixedTheta}
\alias{GetCountUnfixedTheta}
\title{Get Count of Unfixed Thetas}
\usage{
GetCountUnfixedTheta()
}
\value{
integer
}
\description{
Get Count of Unfixed Thetas
}
\keyword{internal}
