% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCountEta}
\alias{GetCountEta}
\title{Get Count of Etas}
\usage{
GetCountEta()
}
\value{
integer, number of eta parameters, or NA if file not found
}
\description{
Get Count of Etas
}
