% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_stats.R
\name{NMIDStat}
\alias{NMIDStat}
\title{NONMEM Individual (ID) Statistics}
\usage{
NMIDStat(NMTable)
}
\arguments{
\item{NMTable}{data.frame of NONMEM data}
}
\value{
list with IDStat data.frame, sortedID logical, sortedDT logical
}
\description{
Computes per-subject statistics including record counts,
DV counts, dosing information, and sorting checks.
}
