% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdf_report.R
\name{PrinTxt}
\alias{PrinTxt}
\title{Print Text at Position}
\usage{
PrinTxt(Row, Col, Text, Cex = 0.8)
}
\arguments{
\item{Row}{numeric, row position}

\item{Col}{numeric, column position}

\item{Text}{character, text to print}

\item{Cex}{numeric, character expansion factor}
}
\description{
Prints text at a specific row and column position on the PDF page.
}
