% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCurModelName}
\alias{GetCurModelName}
\title{Get Current Model Name from Working Directory}
\usage{
GetCurModelName()
}
\value{
character, the model name
}
\description{
Detects model name by finding the .xml file in the current directory.
Falls back to extracting from folder name if no .xml file is found.
}
\keyword{internal}
