% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_tests.R
\name{ResTest}
\alias{ResTest}
\title{Residual Count Test Using Binomial Distribution}
\usage{
ResTest(
  ResTab,
  TestCols = c("WRES", "CWRE", "IWRE"),
  ZVals = c(0, 1, 2, 3),
  PctVals = c(0.005, 0.01, 0.025, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)
)
}
\arguments{
\item{ResTab}{data.frame with residual columns}

\item{TestCols}{character vector of column names to test}

\item{ZVals}{numeric vector of Z-value thresholds}

\item{PctVals}{numeric vector of percentile values}
}
\value{
matrix of test results
}
\description{
Tests whether residual counts at various Z-value thresholds follow
expected binomial distributions.
}
