% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{SESuccess}
\alias{SESuccess}
\title{Check Standard Error Success}
\usage{
SESuccess()
}
\value{
logical, TRUE if standard errors were computed successfully
}
\description{
Check Standard Error Success
}
