% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCountEps}
\alias{GetCountEps}
\title{Get Count of Epsilons}
\usage{
GetCountEps()
}
\value{
integer, number of epsilon parameters, or NA if file not found
}
\description{
Get Count of Epsilons
}
