% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_stats.R
\name{NMVarStat}
\alias{NMVarStat}
\title{NONMEM Variable Statistics}
\usage{
NMVarStat(NMTable)
}
\arguments{
\item{NMTable}{data.frame of NONMEM data}
}
\value{
matrix with variable statistics
}
\description{
Computes statistics for each variable in a NONMEM dataset including
counts of NA, zero, positive values, unique values, and functional
dependency checks.
}
