% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{MinSuccess}
\alias{MinSuccess}
\title{Check Minimization Success}
\usage{
MinSuccess()
}
\value{
logical, TRUE if minimization was successful
}
\description{
Check Minimization Success
}
