% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pdf_report.R
\name{ClosePDF}
\alias{ClosePDF}
\title{Close PDF Output}
\usage{
ClosePDF()
}
\description{
Closes the current PDF device.
}
