% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pairs.R
\name{EBEpair}
\alias{EBEpair}
\title{EBE Pair Plot}
\usage{
EBEpair(tabEta, RunNumber, each = FALSE)
}
\arguments{
\item{tabEta}{data.frame with ID, covariates, and ETA columns}

\item{RunNumber}{character, model/run identifier}

\item{each}{logical, if TRUE creates separate plots per covariate}
}
\description{
Creates pairs plot of ETAs vs covariates with histograms and correlations.
}
