% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetCountObs}
\alias{GetCountObs}
\title{Get Count of Observations}
\usage{
GetCountObs()
}
\value{
integer, total number of observation records
}
\description{
Get Count of Observations
}
