% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nm_read.R
\name{GetAICc}
\alias{GetAICc}
\title{Get Corrected AIC (AICc)}
\usage{
GetAICc()
}
\value{
numeric, the corrected AIC value
}
\description{
Get Corrected AIC (AICc)
}
